#include <wx/filename.h>
#include "filetree.h"
#include "droptarget.h"
#include "uthread.h"

enum
{
	O_TREE = wxID_HIGHEST,
};

BEGIN_EVENT_TABLE(UFiletree, wxTreeCtrl)
	EVT_TREE_BEGIN_DRAG(O_TREE, UFiletree::OnDragBegin)
	EVT_TREE_END_DRAG(O_TREE, UFiletree::OnDragEnd)
	EVT_TREE_ITEM_EXPANDING(O_TREE, UFiletree::OnItemExpanding)
	EVT_TREE_ITEM_COLLAPSED(O_TREE, UFiletree::OnItemCollapsed)
	EVT_TREE_ITEM_COLLAPSING(O_TREE, UFiletree::OnItemCollapsing)
	EVT_TREE_ITEM_ACTIVATED(O_TREE, UFiletree::OnItemActivated)
	EVT_TREE_BEGIN_LABEL_EDIT(O_TREE, UFiletree::OnItemBeginRename)
	EVT_TREE_END_LABEL_EDIT(O_TREE, UFiletree::OnItemEndRename)
	EVT_LEAVE_WINDOW(UFiletree::OnLeaveWindow)
END_EVENT_TABLE()

UFiletree::UFiletree(wxWindow *oParent, wxWindowID iID, wxTextCtrl *oLocation, MainWindow *oParentWindow)
: wxTreeCtrl(oParent, O_TREE, wxDefaultPosition, wxDefaultSize, wxTR_HAS_BUTTONS | wxTR_TWIST_BUTTONS | wxTR_DEFAULT_STYLE | wxTR_ROW_LINES | wxTR_MULTIPLE)
{
	bSeeHidden = false;
	oConfig = (wxConfig *)wxConfig::Get();
	txtLocation = oLocation;
	oParentFrame = oParentWindow;
	SetIndent(14);
	SetSpacing(8);
	SetBackgroundColour(wxColour(240, 240, 240));
	SetDropTarget(new UDropTarget(this));
	selectTreeItem(GetRootItem());
	
	createThread();
}

void UFiletree::dropFiles(std::vector<wxString> sFiles, wxCoord oX, wxCoord oY)
{
	wxTreeItemId iItem = HitTest(wxPoint(oX, oY));
	wxString sDestination = getFullpath(iItem);
	if (wxFile::Exists(sDestination))
	{
		sDestination = sDestination.Mid(0, sDestination.Find('/', true));
	}
	if (wxDirExists(sDestination))
	{
		for (unsigned int i = 0; i < sFiles.size(); i++)
		{
			move(sFiles[i], sDestination);
		}
	}
}

void UFiletree::OnLeaveWindow(wxMouseEvent &oEvent)
{
	if (oEvent.LeftIsDown())
	{
		wxFileDataObject oFiles;
		
		wxArrayTreeItemIds iSelected;
		GetSelections(iSelected);
		for (unsigned int i = 0; i < iSelected.GetCount(); i++)
		{
			oFiles.AddFile(getFullpath(iSelected[i]));
		}
		
		oDropSource = new UDropSource(this);
		oDropSource->SetData(oFiles);
		
		//printf("Dragging something off the table!\n");
		wxString sString;
		switch ( oDropSource->DoDragDrop(wxDrag_DefaultMove) )
		{
			case wxDragError:   sString = _T("Error!");    break;
			case wxDragNone:    sString = _T("Nothing");   break;
			case wxDragCopy:    sString = _T("Copied");    break;
			case wxDragMove:    sString = _T("Moved");     break;
			case wxDragCancel:  sString = _T("Cancelled"); break;
			default:            sString = _T("Huh?");      break;
	        }
		wxSetCursor(0);
	}
}

void UFiletree::createThread()
{
	oThread = new UThread(this);
	oThread->Create();
	oThread->Run();
}

bool UFiletree::deleteItem()
{
	if (!somethingSelected())
	{
		return false;
	}
	
	std::vector<wxTreeItemId> iItems = getTreeSelections();
	
	// Security measures.
	// Also, what are we going to view if we delete everything we see?
	// Going to the upper level directory would probably do nothing but confuse the user.
	// (Besides, if I'd accidentally do such a stupid thing and the program was arrogant enough to list that as a feature... I'd strangle a puppy.)
	for (unsigned int i = 0; i < iItems.size(); i++)
	{
		if (GetRootItem() == iItems[i])
		{
			return false;
		}
	}
	
	wxMessageDialog oDialog(this, _("Are you sure you want to delete these items?"), _("Delete items"), wxYES_NO);
	int iResult = oDialog.ShowModal();
	if (iResult == wxID_YES)
	{
		unsigned int iOKs = 0;
		for (unsigned int i = 0; i < iItems.size(); i++)
		{
			if (!itemExists(iItems[i]))
			{
				continue;
			}
			wxTreeItemId iParent = GetItemParent(iItems[i]);
			wxString sItem = getFullpath(iItems[i]);
			wxString sRunString;
			if (wxDirExists(sItem))
			{
				sRunString = _("rm -rf \"");
			}
			else
			{
				sRunString = _("rm -f \"");
			}
			sRunString += sItem + _("\"");
			wxArrayString sStdOutput;
			wxArrayString sStdErrors;
			iResult = wxExecute(sRunString, sStdOutput, sStdErrors);
			if (iResult == 0)
			{
				wxTreeItemId iNext;
				wxTreeItemId iPrev;
				wxString sParentDir;
				sParentDir = getFullpath(iParent);
				
				if (itemExists(iItems[i]))
				{
					iNext = GetNextSibling(iItems[i]);
					iPrev = GetPrevSibling(iItems[i]);
					
					if (itemExists(iNext))
					{
						//selectTreeItem(iNext);
					}
					else if (itemExists(iPrev))
					{
						selectTreeItem(iPrev);
					}
					else
					{
						selectTreeItem(iParent);
					}
				}
				
				wxDir oDir(sParentDir);
				if (!oDir.HasFiles() && !oDir.HasSubDirs())
				{
					Collapse(iParent);
					SetItemHasChildren(iParent, false);
				}
				iOKs++;
				
				logMessage(_("File(s) deleted."));
			}
			else
			{
				logMessage(sStdErrors[0]);
			}
		}
		if (iOKs == iItems.size())
		{
			return true;
		}
	}
	return false;
}

void UFiletree::logMessage(const wxString &sMessage)
{
	oParentFrame->SetStatusText(sMessage);
	this->SetFocus();
}

void UFiletree::OnDragBegin(wxTreeEvent &oEvent)
{
	iMovingID = oEvent.GetItem();
	oEvent.Allow();
}

void UFiletree::OnDragEnd(wxTreeEvent &oEvent)
{
	wxString sOldItem;
	wxString sDestination;
	wxString sRunString;
	if (wxDirExists(getFullpath(oEvent.GetItem())))
	{
		sOldItem = getFullpath(this->iMovingID);
		sDestination = getFullpath(oEvent.GetItem());
		if (sOldItem == _("!"))
		{
			return;
		}
		else if (sOldItem == sDestination + _("/") + GetItemText(this->iMovingID))
		{
			return;
		}
		move(sOldItem, sDestination);
	}
}

wxTreeItemId UFiletree::addItem(wxTreeItemId iDirID, const wxString &sFullpath)
{
	wxTreeItemId iTempID;
	wxString sItem = wxFileName::FileName(sFullpath).GetFullName();
	int iImage;
	if (wxDirExists(sFullpath)) // It's a directory!
	{
		if (wxFile::Access(sFullpath, wxFile::read))
		{
			iTempID = AppendItem(iDirID, sItem, (isSymlink(sFullpath) ? 6 : 1));
			wxDir oDir(sFullpath);
			if (oDir.HasFiles() || oDir.HasSubDirs())
			{
				SetItemHasChildren(iTempID, true);
			}
		}
		else
		{
			iTempID = AppendItem(iDirID, sItem, 3);
		}
	}
	else // It's a file! (Or at least not a directory.)
	{
		if (isSymlink(sFullpath))
		{
			iImage = (isExecutable(sFullpath) ? 5 : 4);
		}
		else
		{
			if (isExecutable(sFullpath))
			{
				iImage = 2;
			}
			else
			{
				if (wxFile::Access(sFullpath, wxFile::read))
				{
					iImage = 0;
				}
				else
				{
					iImage = 7;
				}
			}
		}
		
		iTempID = AppendItem(iDirID, sItem, iImage);
	}
	SetItemTextColour(iTempID, wxColour(70, 70, 70));
	addValid(iTempID);
	return iTempID;
}

void UFiletree::OnItemExpanding(wxTreeEvent &oEvent)
{
	wxString sDir(getFullpath(oEvent.GetItem()));
	
	ReloadDir(oEvent.GetItem(), /*sDir, */false);

	oParentFrame->treeFit();
}

void UFiletree::OnItemCollapsing(wxTreeEvent &oEvent)
{
	wxTreeItemId iItem = oEvent.GetItem();
	wxTreeItemIdValue iDummy;
	wxTreeItemId iCurrent = GetFirstChild(iItem, iDummy);
	while (iCurrent.IsOk())
	{
		delValid(iCurrent);
		
		iCurrent = GetNextChild(iItem, iDummy);
	}
}

void UFiletree::OnItemCollapsed(wxTreeEvent &oEvent)
{
	wxString sFullpath(getFullpath(oEvent.GetItem()));
	wxDir oDir(sFullpath);
	
	if (itemExists(oEvent.GetItem()))
	{
		DeleteChildren(oEvent.GetItem());
		if (oDir.HasFiles() || oDir.HasSubDirs())
		{
			SetItemHasChildren(oEvent.GetItem(), true);
		}
	}
	
	oParentFrame->treeFit();
}

void UFiletree::OnItemActivated(wxTreeEvent &oEvent)
{
	Activate(oEvent.GetItem());
}

void UFiletree::OnItemBeginRename(wxTreeEvent &oEvent)
{
	sTempItemLabel = GetItemText(oEvent.GetItem());
}

void UFiletree::OnItemEndRename(wxTreeEvent &oEvent)
{
	if (!Rename(oEvent.GetItem(), oEvent.GetLabel()))
	{
		SetItemText(oEvent.GetItem(), sTempItemLabel);
	}
}

void UFiletree::ReloadDir(wxTreeItemId iItemID, bool bCheckOnly)
{
	wxString sDir(getFullpath(iItemID));
	
	wxDir oDir(sDir);
	if (oDir.HasFiles() || oDir.HasSubDirs())
	{
		// wxMutexGuiEnter();
		if (GetChildrenCount(iItemID) == 0)
		{
			SetItemHasChildren(iItemID, true);
			if (bCheckOnly)
			{
				return;
			}
		}
		// wxMutexGuiLeave();
	}
	
	wxString sItem;
	std::vector<wxString> sItemArray;
	bool bCont = oDir.GetFirst(&sItem);
	while (bCont)
	{
		sItemArray.push_back(sItem);
		
		bCont = oDir.GetNext(&sItem);
	}

	for (unsigned int i = 0; i < sItemArray.size(); i++)
	{
		// wxMutexGuiEnter();
		if ((bSeeHidden || sItemArray[i].Left(1) != _(".")) && !itemExists(sDir + _("/") + sItemArray[i]))
		{
			addItem(iItemID, sDir + _("/") + sItemArray[i]);
		}
		// wxMutexGuiLeave();
	}
	
	sItem = _("");
	std::vector<wxTreeItemId> iDeletedItems;
	// wxMutexGuiEnter();
	wxTreeItemId iCurrent = GetRootItem();
	while (iCurrent = GetNextVisible(iCurrent))
	{
		sItem = getFullpath(iCurrent);
		if (!wxDirExists(sItem) && !wxFile::Exists(sItem))
		{
			iDeletedItems.push_back(iCurrent);
		}
	}
	// wxMutexGuiLeave();
	
	for (unsigned int i = 0; i < iDeletedItems.size(); i++)
	{
		// wxMutexGuiEnter();
		if (getSingleTreeSelection() == iDeletedItems[i])
		{
			wxTreeItemId iParent = GetItemParent(iDeletedItems[i]);
			wxString sParentDir(getFullpath(iParent));
			
			wxTreeItemId iNext = GetNextSibling(iDeletedItems[i]);
			wxTreeItemId iPrev = GetPrevSibling(iDeletedItems[i]);
			
			if (itemExists(iNext))
			{
				selectTreeItem(iNext);
			}
			else if (itemExists(iPrev))
			{
				selectTreeItem(iPrev);
			}
			else
			{
				selectTreeItem(iParent);
			}
			
			wxDir oDir(sParentDir);
			if (!oDir.HasFiles() && !oDir.HasSubDirs())
			{
				Collapse(iParent);
				SetItemHasChildren(iParent, false);
			}
		}
		if (itemExists(iDeletedItems[i]))
		{
			delValid(iDeletedItems[i]);
			Delete(iDeletedItems[i]);
		}
		// wxGuiLeave();
	}
	
	// wxGuiEnter();
	SortChildren(iItemID);
	// wxGuiLeave();
}

void UFiletree::SetRoot(const wxString &sDir, bool bSkipLocationText)
{
	if (GetRootItem())
	{
		if (sDir == GetItemText(GetRootItem()) + _("/") || sDir + _("/") == GetItemText(GetRootItem()))
		{
			return;
		}
	}
	
	delAllValids();
	DeleteAllItems();
	if (!wxFile::Access(sDir, wxFile::read))
	{
		AddRoot(sDir, 3);
	}
	else
	{
		AddRoot(sDir, 1);
		SetItemTextColour(GetRootItem(), wxColour(70, 70, 70));
		SetItemHasChildren(GetRootItem(), true);
		Expand(GetRootItem());
	}
	
	addValid(GetRootItem());
	
	// We do this only to avoid a neverending while-loop,
	// because when we change the value of txtLocation,
	// OnLocationChange() is called, which calls SetRoot
	// again. Which is bad.
	if (!bSkipLocationText)
	{
		txtLocation->SetValue(sDir);
	}
	
	oParentFrame->SetTitle(sDir);
	wxSetWorkingDirectory(sDir);
}

void UFiletree::Activate(wxTreeItemId iItemID)
{
	wxString sItem = getFullpath(iItemID);
	Activate(sItem);
}

void UFiletree::Activate(wxString sItem)
{
	if (wxDirExists(sItem))
	{
		sItem.Replace(_("//"), _("/"));
		SetRoot(sItem, false);
	}
	else
	{
		if (isExecutable(sItem))
		{
			run(sItem);
		}
		else
		{
			wxString sFilename = sItem.Mid(sItem.Find('/', true) + 1);
			int iLoc = sFilename.Find('.', true);
			wxString sEnding = (iLoc != -1 ? sFilename.Mid(iLoc) : sFilename);
			wxString sApp = oConfig->Read(_("/Files/") + sEnding + _("/Run"), oConfig->Read(_("/DefaultApp")));
			wxString sRunString = sApp + _(" \"") + sItem + _("\"");
			run(sRunString, oConfig->Read(_("/Files/") + sEnding + _("/RunInTerminal")) == _("1"));
		}
	}
}

bool UFiletree::addFile()
{
	if (!somethingSelected())
	{
		return false;
	}
	
	wxTreeItemId iItemID = getSingleTreeSelection();
	wxString sFilename;
	wxString sRunString;
	wxString sParentDir;
	wxString sAllegedDir;
	wxTreeItemId iParentID;
	int iResult;
	sFilename = wxGetTextFromUser(_("Enter name of new file"), _("Name"), _(""));
	
	if (sFilename == _(""))
	{
		return false;
	}
	
	sAllegedDir = getFullpath(iItemID);
	if (wxDirExists(sAllegedDir))
	{
		iParentID = iItemID;
		sParentDir = sAllegedDir;
	}
	else
	{
		iParentID = GetItemParent(iItemID);
		sParentDir = getFullpath(iParentID);
	}
	if (!wxFile::Exists(sParentDir + _("/") + sFilename))
	{
		sRunString = _("touch \"") + sParentDir + _("/") + sFilename + _("\"");
		wxArrayString sStdOutput;
		wxArrayString sStdErrors;
		iResult = wxExecute(sRunString, sStdOutput, sStdErrors);
		if (iResult == 0)
		{
			logMessage(_("File added."));
			return true;
		}
		else
		{
			logMessage(sStdErrors[0]);
			return false;
		}
	}
	return false;
}

bool UFiletree::addFolder()
{
	if (!somethingSelected())
	{
		return false;
	}
	
	wxTreeItemId iItemID = getSingleTreeSelection();
	wxString sFoldername;
	wxString sRunString;
	wxString sParentDir;
	wxString sAllegedDir;
	wxTreeItemId iParentID;
	int iResult;
	sFoldername = wxGetTextFromUser(_("Enter name of new folder"), _("Name"), _(""));
	
	if (sFoldername == _(""))
	{
		return false;
	}
	
	sAllegedDir = getFullpath(iItemID);
	if (wxDirExists(sAllegedDir))
	{
		iParentID = iItemID;
		sParentDir = sAllegedDir;
	}
	else
	{
		iParentID = GetItemParent(iItemID);
		sParentDir = getFullpath(iParentID);
	}
	if (!wxDirExists(sParentDir + _("/") + sFoldername))
	{
		sRunString = _("mkdir \"") + sParentDir + _("/") + sFoldername + _("\"");
		wxArrayString sStdOutput;
		wxArrayString sStdErrors;
		iResult = wxExecute(sRunString, sStdOutput, sStdErrors);
		if (iResult == 0)
		{
			logMessage(_("Folder added."));
			return true;
		}
		else
		{
			logMessage(sStdErrors[0]);
			return false;
		}
	}
	return false;
}

bool UFiletree::Rename(wxTreeItemId iItemID, const wxString &sNewName)
{
	wxString sRunString;
	wxString sItem;
	wxString sNewItem;
	bool bResult = false;
	sItem = getFullpath(iItemID);
	sNewItem = sItem.Left(sItem.Length() - GetItemText(iItemID).Length()) + sNewName;
	if (sNewName.Length() > 0)
	{
		bResult = move(sItem, sNewItem);
	}
	return bResult;
}

void UFiletree::GoUp()
{
	wxString sOldPath = txtLocation->GetValue();
	wxString sNewPath;
	
	if (sOldPath.Right(1) == _("/"))
	{
		sOldPath = sOldPath.Left(sOldPath.Length() - 1);
	}
	
	sNewPath = sOldPath.Left(sOldPath.Find('/', true));
	
	if (sNewPath == _(""))
	{
		sNewPath = _("/");
	}

	if (sNewPath == sOldPath + _("/"))
	{
		return;
	}
	
	SetRoot(sNewPath, false);
	SetFocus();
}

void UFiletree::rightPress()
{
	wxTreeItemIdValue iDummy = 0;
	wxString sFullpath;
	wxTreeItemId iItem;
	if (!somethingSelected())
	{
		return;
	}
	iItem = getSingleTreeSelection();
	sFullpath = getFullpath(iItem);
	if (!wxDirExists(sFullpath))
	{
		return;
	}
	wxDir oDir(sFullpath);
	if (oDir.HasFiles() || oDir.HasSubDirs())
	{
		if (IsExpanded(iItem))
		{
			selectTreeItem(GetFirstChild(iItem, iDummy));
		}
		else
		{
			Expand(iItem);
		}
	}
}

void UFiletree::leftPress()
{
	if (!somethingSelected())
	{
		return;
	}
	
	if (IsExpanded(getSingleTreeSelection()))
	{
		CollapseAndReset(getSingleTreeSelection());
	}
	else if (GetItemParent(getSingleTreeSelection()))
	{
		selectTreeItem(GetItemParent(getSingleTreeSelection()));
	}
}

void UFiletree::selectNextItem()
{
	wxTreeItemId iItem;
	if (!somethingSelected())
	{
		return;
	}
	iItem = getSingleTreeSelection();
	if (GetNextVisible(iItem))
	{
		selectTreeItem(GetNextVisible(getSingleTreeSelection()));
	}
}

void UFiletree::selectPrevItem()
{
	selectTreeItem(getPrevID(getSingleTreeSelection()));
}

void UFiletree::selectTreeItem(wxTreeItemId iItem)
{
	UnselectAll();
	SelectItem(iItem);
}

void UFiletree::pressHome()
{
	selectTreeItem(GetRootItem());
}

void UFiletree::pressEnd()
{
	selectTreeItem(GetLastChild(GetRootItem()));
}

void UFiletree::openWith()
{
	wxTreeItemId iItem;
	if (iItem = getSingleTreeSelection())
	{
		wxString sRunString;
		wxString sFullpath;
		sRunString = wxGetTextFromUser(_("Run with..."), _("Runstring:"), _("gvim '%s'"));
		if (sRunString.Find(_("%s")) == -1)
		{
			sRunString += _(" '%s'");
		}
		
		sFullpath = getFullpath(iItem);
		sRunString.Replace(_("%s"), sFullpath);
		wxExecute(sRunString);
	}
}

bool UFiletree::somethingSelected()
{
	wxArrayTreeItemIds iSelections;
	return (GetSelections(iSelections) > 0);
}

void UFiletree::edit()
{
	if (!somethingSelected())
	{
		return;
	}
	EditLabel(getSingleTreeSelection());
}

wxString UFiletree::getFullpath(wxTreeItemId iItem)
{
	if (!itemExists(iItem))
	{
		return wxString(_("!"));
	}
	wxTreeItemId iNewID = iItem;
	wxString sDir = GetItemText(iItem);
	while (iNewID != GetRootItem())
	{
		iNewID = GetItemParent(iNewID);
		sDir = GetItemText(iNewID) + _("/") + sDir;
	}
	sDir.Replace(_("//"), _("/"));
	return sDir;
}

std::vector<wxString> UFiletree::split(const wxString &sString, char cSplit)
{
	std::vector<wxString> sSplitVector;
	wxString sRest = sString;
	int iCount = sString.Freq(cSplit) + 1;
	for (int i = 0; i < iCount; i++)
	{
		sSplitVector.push_back(sRest.BeforeFirst(cSplit));
		sRest = sRest.AfterFirst(cSplit);
	}
	return sSplitVector;
}

bool UFiletree::itemExists(const wxTreeItemId &iItem)
{
	wxTreeItemId iCurrent = GetRootItem();
	
	if (iCurrent == iItem)
	{
		return true;
	}
	
	bool bReturn = false;
	while (iCurrent = GetNextVisible(iCurrent))
	{
		if (iCurrent == iItem)
		{
			bReturn = true;
			break;
		}
	}
	return bReturn;
}

bool UFiletree::itemExists(const wxString &sItem)
{
	std::vector<wxString> sSplit;
	std::vector<bool> bFound;
	wxTreeItemId iSubject;
	wxTreeItemId iCurrent;
	wxTreeItemIdValue iDummy;
	wxString sPath = GetItemText(GetRootItem());
	wxString sRest;
	
	if (sItem.Left(sPath.Length()) != sPath)
	{
		return false;
	}
	
	sRest = sItem.Mid(sPath.Length() + 1);
	sSplit = split(sRest, '/');
	iSubject = GetRootItem();
	for (unsigned int i = 0; i < sSplit.size(); i++)
	{
		bFound.push_back(false);
		iCurrent = GetFirstChild(iSubject, iDummy);
		while (iCurrent.IsOk())
		{
			if (GetItemText(iCurrent) == sSplit[i])
			{
				bFound[i] = true;
				iSubject = iCurrent;
				break;
			}
			iCurrent = GetNextChild(iSubject, iDummy);
		}
	}
	
	for (unsigned int i = 0; i < bFound.size(); i++)
	{
		if (!bFound[i])
		{
			return false;
		}
	}
	return true;
}

// GetPrevVisible(iItem) doesn't seem to work (2.6.3 - 2006-06-09), that's why we have this one.
wxTreeItemId UFiletree::getPrevID(const wxTreeItemId &iItem)
{
	wxTreeItemId iCurrent = GetRootItem();
	
	if (iItem == iCurrent)
	{
		return iItem;
	}
	
	while (iItem != GetNextVisible(iCurrent))
	{
		iCurrent = GetNextVisible(iCurrent);
	}
	return iCurrent;
}

std::vector<wxTreeItemId> UFiletree::getTreeSelections()
{
	wxArrayTreeItemIds iSelections;
	GetSelections(iSelections);
	
	std::vector<wxTreeItemId> iNewSelections;
	for (unsigned int i = 0; i < iSelections.GetCount(); i++)
	{
		iNewSelections.push_back(iSelections[i]);
	}
	return iNewSelections;
}

wxTreeItemId UFiletree::getSingleTreeSelection()
{
	wxArrayTreeItemIds iSelections;
	GetSelections(iSelections);
	if (iSelections.GetCount() > 0)
	{
		return iSelections[iSelections.GetCount() - 1];
	}
	wxTreeItemId iSpareItem;
	return iSpareItem;
}

void UFiletree::run(const wxString &sCommand)
{
	run(sCommand, false);
}

void UFiletree::run(const wxString &sCommand, bool bTerminal)
{
	if (!bTerminal)
	{
		wxExecute(sCommand);
		return;
	}
	
	oParentFrame->oProcessBox->setCommand(sCommand);
	
	if (!oParentFrame->oProcessBox->IsShown())
	{
		oParentFrame->oProcessBox->Show(true);
		oParentFrame->btnProcessClose->Show(true);
		oParentFrame->oMainSizer->Add(oParentFrame->oProcessBox, 0, wxGROW);
		oParentFrame->oMainSizer->Add(oParentFrame->btnProcessClose, 0, wxALIGN_CENTER, wxALIGN_CENTER);
		oParentFrame->oMainSizer->SetItemMinSize(oParentFrame->oProcessBox, 10, 120);
		oParentFrame->oMainSizer->Layout();
		oParentFrame->btnProcessClose->SetFocus();
	}
	
	oParentFrame->oProcessBox->run();
}

bool UFiletree::isValid(wxTreeItemId iItem)
{
	for (unsigned int i = 0; i < iValidItems.size(); i++)
	{
		if (iValidItems[i] == iItem)
		{
			return true;
		}
	}
	return false;
}

void UFiletree::addValid(wxTreeItemId iItem)
{
	for (unsigned int i = 0; i < iValidItems.size(); i++)
	{
		if (iValidItems[i] == iItem)
		{
			return;
		}
	}
	iValidItems.push_back(iItem);
}

void UFiletree::delValid(wxTreeItemId iItem)
{
	std::vector<wxTreeItemId> iNewItems;
	for (unsigned int i = 0; i < iValidItems.size(); i++)
	{
		if (iValidItems[i] != iItem)
		{
			iNewItems.push_back(iValidItems[i]);
		}
	}
	iValidItems = iNewItems;
}

void UFiletree::delAllValids()
{
	std::vector<wxTreeItemId> iNewItems;
	iValidItems = iNewItems;
}

bool UFiletree::move(const wxString &sOriginal, const wxString &sDestination)
{
	wxString sRunString = _("mv \"") + sOriginal + _("\" \"") + sDestination + _("\"");
	wxArrayString sStdOutput;
	wxArrayString sStdErrors;
	long iResult = wxExecute(sRunString, sStdOutput, sStdErrors);
	if (iResult == 0)
	{
		logMessage(_("File(s) moved."));
		return true;
	}
	else
	{
		logMessage(sStdErrors[0]);
		return false;
	}
}

bool UFiletree::isExecutable(const wxString &sFilename)
{
	return !(access((const char *)sFilename.mb_str(), X_OK) < 0);
}

bool UFiletree::isSymlink(const wxString &sFilename)
{
	struct stat oStat;
	lstat((const char *)sFilename.mb_str(), &oStat);
	return (S_ISLNK(oStat.st_mode) ? true : false);
}

